var kSidebarPrefixTag = "#nav-";

function ExpandNavigationList()
{
	var navDiv = document.getElementById('navigationView');	// Get the view object.
	if ( !navDiv ) return;
	
	var links = navDiv.getElementsByTagName("li");

	var i;
	for (i=0; i < links.length; i++)
	{
		var link = links[i];

		if ( link.className == "closed hasChildren" )
		{
			link.className = "open hasChildren";
		}
	}
}

function CollapseNavigationList()
{
	var navDiv = document.getElementById('navigationView');	// Get the view object.
	if ( !navDiv ) return;
	
	var links = navDiv.getElementsByTagName("li");

	var i;
	for (i=0; i < links.length; i++)
	{
		var link = links[i];

		if ( link.className == "open hasChildren" )
		{
			link.className = "closed hasChildren";
		}
	}
}





/*******************************************************************************
 SidebarList
 
 This function returns an object representing the visible list container in the
 navigation view. If no products are selected, then we return the div with
 id "navigationView". Otherwise, we return the outermost "ul" list object whose
 classname corresponds to the current product name.
 ******************************************************************************/
function SidebarList()
{
	var navDiv = document.getElementById("navigationView");
	if ( !navDiv ) return null;
	
	
	// Get the list that contains the product.
	var links = navDiv.getElementsByTagName("a");
	return links;
}



/*
 Called when the user clicks the next topic button.
*/
function ShowNextTopic()
{
	// Get the currently-displayed list in the sidebar.
	var links = SidebarList();
	if ( !links ) return;

	// Get all elements in "navDiv".
	var selectedLinkDiv = null;
	var firstLinkDiv = null;
	for (var i=0; i < links.length; i++)
	{
		var linkDiv = links[i];
		if ( linkDiv.hash )
		{
			// Record the first link div, for reference.
			if ( !firstLinkDiv )
			{
				firstLinkDiv = linkDiv;
			}
		
			if ( linkDiv.className == "select" )
			{
				// Record this object.
				selectedLinkDiv = linkDiv;
			}
			else
			{
				// If we've already found the link...
				if ( selectedLinkDiv )
				{
					LoadHashString( linkDiv.hash );
					return;
				}
			}
		}
	}
	
	// If we get here, then no links are selected, so we need to select the first.
	if ( !selectedLinkDiv )
	{
		if ( firstLinkDiv )
		{
			LoadHashString( firstLinkDiv.hash );
		}
	}
}


function ShowPreviousTopic()
{
	// Get the currently-displayed list in the sidebar.
	var links = SidebarList();
	if ( !links ) return;
	
	// Get all elements in "navDiv".
	var prevDiv = null;
	var lastLinkDiv = null;
	for (var i=0; i < links.length; i++)
	{
		var linkDiv = links[i];
		if ( linkDiv.hash )
		{
			// Record final link, just in case nothing is selected.
			lastLinkDiv = linkDiv;
			
			if ( linkDiv.className == "select" )
			{
				// We need to highlight prevDiv.
				if ( prevDiv )
				{
					LoadHashString( prevDiv.hash );
				}
				return;
			}
			else
			{
				prevDiv = linkDiv;
			}
		}
	}
	
	// If we get here, then no links are selected, so we select the last.
	if ( lastLinkDiv )
	{
		LoadHashString( lastLinkDiv.hash );
	}
}







/*******************************************************************************
 Toggles a list item open or closed.
*******************************************************************************/
function toggle( theEvent )
{
	if ( !theEvent ) return;
	
	//
	// Stop the event propagating up, in case the clicked object is embedded
	// inside another object that can also respond to this event.
	//
	if ( theEvent.stopPropagation )
	{
		theEvent.stopPropagation();
	}
	else
	{
		theEvent.cancelBubble = true;
	}

	//
	// Get the element that was clicked.
	// Normally, we would simply call:
	//
	//     theDiv = theEvent.target;
	//
	// But this isn't supported by Internet Explorer which, apparent,
	// doesn't have the property "target". Instead, we have to use
	// "srcElement" instead. I found out this piece of advice from:
	// http://annevankesteren.nl/2005/07/dom-3-events
	//
    var theDiv = theEvent.target ? theEvent.target : theEvent.srcElement;
    											// I.E. Workaround.
	if ( !theDiv ) return;
	
	// The object might be an embedded div - e.g., the disclosure arrow div.
	// We therefore need to move up in the embedding hierarchy, until we find
	// the "LI" tagname (always UPPERCASE)
	while ( theDiv && theDiv.tagName != "LI" )
	{
		theDiv = theDiv.parentNode;
	}
	if ( !theDiv ) return;						// In case no "header" exists.
		
	if ( theDiv.className == "closed hasChildren" )
	{
		theDiv.className = "open hasChildren";
	}
	else if ( theDiv.className == "open hasChildren" )
	{
		theDiv.className = "closed hasChildren";
	}
	
	return;
}

/*******************************************************************************
 LoadHashTag
 JavaScript function to allow a clicked URL to be loaded into the mail frame.
 The "URL" should have the form of a hashtag, e.g., "#MyPage" will result in
 the URL "MyPage.html" being loaded.
 ******************************************************************************/
function LoadHashTag( theEvent )
{
	if ( !theEvent ) return;
	
	//
	// Stop the event propagating up, in case the clicked object is embedded
	// inside another object that can also respond to this event.
	//
	if ( theEvent.stopPropagation )
	{
		theEvent.stopPropagation();
	}
	else
	{
		theEvent.cancelBubble = true;
	}

	//
	// Get the element that was clicked.
	// Normally, we would simply call:
	//
	//     theDiv = theEvent.target;
	//
	// But this isn't supported by Internet Explorer which, apparent,
	// doesn't have the property "target". Instead, we have to use
	// "srcElement" instead. I found out this piece of advice from:
	// http://annevankesteren.nl/2005/07/dom-3-events
	//
    var theDiv = theEvent.target ? theEvent.target : theEvent.srcElement; // I.E. Workaround.
	if ( !theDiv ) return;
	
	// Get the hash and remove the first character, e.g., "#nav-test" -> "test"
	if ( theDiv.hash )
	{
		var len = theDiv.hash.length;
		var startPos = kSidebarPrefixTag.length + theDiv.hash.indexOf(kSidebarPrefixTag);
		var shortURL = theDiv.hash.substring(startPos, len) + ".html";
		LoadURL( shortURL );

		// Ensure that the clicked link is highlighted immediately, so the
		// user doesn't have to move the mouse in order to see the highlight.
		// (This isn't strictly necessary, as LoadURL() does trigger an
		// embedded object "onLoad" handler that does - eventually - update
		// the sidebar. However, that doesn't cause the link to be redrawn
		// until after the user moves the mouse, which doesn't look nice.)
		SelectSidebarLink( theDiv.hash );
	}
	return;
}


function LoadHashString( hashString )
{
	var len = hashString.length;
	var startPos = kSidebarPrefixTag.length + hashString.indexOf(kSidebarPrefixTag);
	var shortURL = hashString.substring(startPos, len) + ".html";
	LoadURL( shortURL );

	// Ensure that the clicked link is highlighted immediately, so the
	// user doesn't have to move the mouse in order to see the highlight.
	// (This isn't strictly necessary, as LoadURL() does trigger an
	// embedded object "onLoad" handler that does - eventually - update
	// the sidebar. However, that doesn't cause the link to be redrawn
	// until after the user moves the mouse, which doesn't look nice.)
	SelectSidebarLink( hashString );
}

/*******************************************************************************
 LoadURL
 We expect a URL string such as "OpenCrystal.html" or "Glossary.html#Eta".
 We prefix this with "pages/".
 ******************************************************************************/
function LoadURL( urlString )
{
	if ( !urlString ) return;
	
	// Insert an "object" inside the "conventView" div.
	// Note: the text between the <object> and </object> is an alternate text,
	// for browsers that do not support this tag - or can be used to display
	// error information if a clicked link was not found.
	
	var introString = '<object id="viewer" name="viewer" type="text/html" onload="LoadObjectHandler()" data="';
	var relativeURL = 'pages/'+urlString;	// (Assume all pages are in pages/)
	var errorString = "<p>&nbsp;&nbsp;&nbsp;Sorry, an error occurred - the linked file could not be opened.</p>";
	var closeString = '<\/object>';
	
	var contentView = document.getElementById('contentView');
	if ( contentView )
	{
	    contentView.innerHTML = introString + relativeURL + '">' + errorString + closeString;
	}

	return false;
}

/*******************************************************************************
 LoadObjectHandler
 
 This function has two purposes:
 
 1. It lets us update the sidebar so that the appropriate item is shown
 	selected, to match the data object's document title.
 
 	e.g., say we have loaded the linked document "OpenCrystal.html".
	We would get the document's full URL, and trim this to get the "OpenCrystal"
 	stem. We would then add a hash symbol (#) to the beginning, and pass this
	to the function SelectSidebarLink.

 2.	It also lets us reset the displayed URL in the browser's address field.
 	This is vital, because links clicked inside the embedded document won't
 	get their URLs shown. It is also important, because clicking on a sidebar
 	link might result in invalid URLs being shown, e.g., "index.html#Overview",
 	instead of "index.html?topic=Overview" (which is valid).
 
 Last revised by DCP: 03-Jan-2014 to workaround IE 9 and earlier which doesn't
                                  support the history.pushState method.
 ******************************************************************************/
function LoadObjectHandler()
{
	/* debug */
//	console.log("object loaded...");
	var documentTitle = "CrystalDiffract Help"
	var fileStem = "";

	//--------------------------------------------------------------------------
	// Get the correct fileStem and title for the embedded document object.
	// (If a new web page is loaded into the object, then the browser's URL
	// still shows the old web page.)
	//--------------------------------------------------------------------------
	var viewerDiv = document.getElementById('viewer');	// Get the view object.
	if ( viewerDiv )
	{
		var doc = viewerDiv.contentDocument;			// Embedded document.
		if ( doc )
		{
			var docURL = doc.URL;						// Embedded doc's URL.
			documentTitle = doc.title;					// Embedded doc't title.
			
			//
			// Extract the embedded document's fileStem: this is the filename
			// minus the ".html" extension. To do this, we shall cycle back
			// through its URL string, searching for the first "/".
			//
			var directoryDelimitor = "/";
			var isWindows = navigator.platform.toUpperCase().indexOf('WIN') >= 0;
			if ( isWindows )
			{
				directoryDelimitor = "\\";
			}
			
			var posHTML = docURL.indexOf('.html');		// Position of file extension.
			var posStart = posHTML - 1;
			while (posStart > 0)
			{
				var c = docURL.substring(posStart-1, posStart);
				
				if ( c == directoryDelimitor )
				{
					break;
				}
				else
				{
					posStart--;
				}
			}
			
			fileStem = docURL.substring(posStart,posHTML);	// Remove file extension.
			
			//
			// We are now in a position to select the corresponding sidebar
			// link: we simply pre-pend a hash symbol to the fileStem, and send 
			// this to our SelectSidebarLink() function.
			//
			var hashTag = kSidebarPrefixTag + fileStem;	// Add a hash to the beginning....
			SelectSidebarLink( hashTag );	// Ensure the appropriate link is selected.
		}
	}

	//--------------------------------------------------------------------------
	//	We now need to set the document's url - as displayed in the browser's
	//	search bar. There are two reasons for this:
	//
	//	1.	If the user clicks a sidebar link, a hash-tag character will be
	//		shown in theURL, e.g.,
	//
	//			http://pathToHelp/index.html#NewPage
	//
	//		or:
	//
	//			http://pathToHelp/index.html?topic=OldPage#NewPage
	//
	//		Neither of which will work - even if we edit our page-loading
	//		handler to decode the string (it just doesn't get called, as the
	//		browser seems to stall at this non-existent "hyperlink").
	//
	//		Instead, we need to set the display to something that WILL work, e.g.,
	//
	//			http://pathToHelp/index.html?topic=NewPage
	//
	//	2.	If the user clicks a link *inside* our embedded object HTML, then
	//    	the displayed URL won't change, e.g.,
	//
	//			it will show:	http://pathToHelp/index.html?topic=OldPage
	//
	//			instead of:		http://pathToHelp/index.html?topic=NewPage
	//
	//	Our solution is to take advantage of the history method, "pushState":
	//	we can specify a document title, partial filename, and complete url.
	//--------------------------------------------------------------------------
	var fullURL = window.document.URL;
	var urlStem = "";
	var pos		= 0;

	// Test for a key-value pair, e.g., "http://pathToHelp/index.html?topic=OldPage".
	pos = fullURL.indexOf('?topic=');
	if ( pos > 0 )
	{
		urlStem = fullURL.substring(0, pos+7);	// Get string to end of "?topic=".
		fullURL = urlStem + fileStem;			// Append the *correct* embedded document's fileStem.
	}
	else
	{
		// Test for presence of a hash-tag, e.g., "http://pathToHelp/index.html#
		pos = fullURL.indexOf('#');
		if ( pos > 0 )	// If we have a hash-tag link, e.g., "index.html#NewPage
		{
			urlStem = fullURL.substring(0, pos);	// Get string up to, but not including, the "#"
			fullURL = urlStem + "?topic=" + fileStem;	// Append "?topic=" and *correct* embedded document's fileStem
		}
	}
	
	// Push the correct url string to the browser display.
	if ( urlStem.length > 0 )
	{
		// Check that "pushState" is supported - requires Internet Explorer 10 or later.
		if ( history.pushState )
		{
			history.pushState( documentTitle, fileStem, fullURL );
		}
		else
		{
			// Force the browser to load the correct URL.
			if ( window.document.URL != fullURL )
			{
				window.location.replace( fullURL );
			}
		}
	}
	
}

/*******************************************************************************
 SelectSidebarLink
 Updates the sidebar so that the corresponding element (with hash tag) is
 shown selected, and its group is open.
 ******************************************************************************/
function SelectSidebarLink( hashTag )
{
	var theDiv = document.getElementById("navigationView");
	if ( !theDiv ) return;
	
	var links = theDiv.getElementsByTagName("a");
	
	// Get all elements in "TheDiv".
	var i;
	for (i=0; i < links.length; i++)
	{
		var link = links[i];
		if ( link.hash )
		{
			if ( link.hash == hashTag )
			{
				// If this isn't highlighted, then highlight it.
				if ( link.className != "select" )
				{
					link.className = 'select';
				}
				
				// Ensure that its parent is open.
				theDiv = link.parentNode;

				while ( theDiv && theDiv.id != "navigationView")
				{
					if ( theDiv.tagName == "LI" )
					{
						if ( theDiv.className == "closed hasChildren" )
						{
							theDiv.className = "open hasChildren";
						}
					}
					
					theDiv = theDiv.parentNode;
				}
				
			}
			else
			{
				// Remove any hilite
				link.className = '';
			}
		}
	}
}


/*******************************************************************************
 LoadHelpTopicsHandler
 
 This is the page load handler for our online help "index" page.
 It is designed to look for a hash-tag, and to use this to load the appropriate
 web page in our viewer pane.
 
 For example, if the user clicks on the "NewCrystal" link in our sidebar, we get
 a URL of the form: "http://pathToHelpFolder/index.html#NewCrystal". This
 script will extract the "NewCrystal" and pass this to the LoadURL() function.

 Note for future update: We probably ought to be able to cope with second hash,
 in case need to scroll, e.g.,
 "http://pathToHelpFolder/index.html#Glossary#AsymmetricUnit"
*******************************************************************************/
/*
function LoadHelpTopicsHandler()
{
	var string = window.document.URL;
	var parms = string.split('#');

	if ( parms.length > 1 )
	{
		var documentTitle = parms[1] + ".html";
		
		if ( parms.length > 2 )
		{
			documentTitle = parms[1] + ".html#" + parms[2];
		}
		
		LoadURL( documentTitle )
	}
	else
	{
		// Fall-through, in case no parameters specified:
		LoadURL( "Default.html" );
	}
	return;
}
*/

/*******************************************************************************
 Old handler code below:
 
 The handler looks for any parameters that are passed with the URL, e.g.,
 "http://crystalmaker.com/support/advice.html?topic=27"
 
 N.B., this would require us to enter something like:
 http://pathToHelpFolder/index.html?topic=NewCrystal
 
 We might consider something like:
 http://pathToHelpFolder/index.html?topic=Glossary&bookmark=Eta
*******************************************************************************/
function LoadHelpTopicsHandler()
{
	var query = window.location.search.substring(1);
	var parms = query.split('&');

	//console.log("query = " + query );
	//console.log("parms = " + parms );
	//console.log("parmslength = " + parms.length );

	for (var i=0; i < parms.length; i++)
	{
		var pos = parms[i].indexOf('=');
		if (pos > 0)
		{
			var key = parms[i].substring(0,pos);
			var val = parms[i].substring(pos+1);

			//console.log("key = " + key );
			//console.log("val = " + val );

			if ( key == "topic" )		// look for the "topic" key.
			{
				var url = val + ".html";
				LoadURL( url );
				return;
			}
		}
	}
	
	// Fall-through, in case no parameters specified:
	LoadURL( "Default.html" );
}




function MacOrWindows()
{
	var ua = navigator.userAgent.toLowerCase();
	if (ua.indexOf("win") != -1)
	{
	    return "Windows";
	}
	else if (ua.indexOf("mac") != -1)
	{
	    return "Mac";
	}
	else
	{
	    return "";
	}
}





